<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Dec 3, 2019
 */

/**
 * Siteworx Cloudlinux Controller.
 *
 * @package    InterWorx
 * @subpackage Ctrl
 */
class Ctrl_Siteworx_Cloudlinux extends Ctrl_Siteworx_Plugin
{

    /**
     * Init.
     */
    protected function _init()
    {
    }

    /**
     * Panel info script.
     */
    public function panelInfoAction()
    {
        $hostname  = Ini::get('iworx', 'hostname');
        $login_url = "https://{$hostname}:2443/siteworx";
        $data = $this->_getPlugin()->getPanelInfo($login_url);
        $Payload = new Payload($data);
        $this->_displayPayload($Payload);
    }

    /**
     * Domains script.
     *
     * @param Form_SW_Cloudlinux_Domains $Form
     */
    public function domainsCommitAction(Form_SW_Cloudlinux_Domains $Form)
    {
        $specified_owner = IW::SW()->getUniqname();

        if ($Form->getValue('name')) {
            $specified_domain = $Form->getValue('name');
        }

        $data = $this->_getPlugin()->getDomains($specified_owner, $specified_domain);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Ui User Info Script.
     *
     * @param Form_NW_Cloudlinux_UserInfo $Form
     */
    public function uiUserInfoCommitAction(Form_NW_Cloudlinux_UserInfo $Form)
    {
        $token_param = $Form->getValue('token');
        $data = $this->_getPlugin()->getUiUserInfo($token_param);
        $this->_displayPayload(new Payload($data));
    }

    /**
     * Returns CL stats
     *
     * returns string $result
     */
    public function clStats()
    {
        $SW = $this->_getSiteWorx();
        $cmd = "/usr/sbin/lveinfo -u {$SW->getUniqname()} --json";
        $Cmd = new Command();
        $Cmd->setCmd($cmd)
            ->exec();
        $result = $Cmd->getOutput();

        //if nothing has happened recently an empty array is returned. During implementation we need to decide what to do here.
        if (empty($result['data'])) {
            return false;
        }

        $result = (array) json_decode($result[0]);
        $result = (array) $result['data'][0];

        //collect stats.
        $stats['number_of_processes'] = $this->_getNumberOfProcesses($result);
        $stats['virtual_memory_usage'] = $this->_getVirtualMemUsage($result);
        $stats['physical_memory_usage'] = $this->_getPhysicalMemUsage($result);
        $stats['cpu_usage'] = $this->_getCpuUsage($result);
        $stats['entry_processes'] = $this->_getEntryProcesses($result);
        $stats['io_operations'] = $this->_getIoOperations($result);
        $stats['io_usage'] = $this->_getIoUsage($result);

        return $stats;
    }

    /**
     * Returns array with average number as key and limit as value.
     *
     * @param array $data
     * @return array
     */
    private function _getNumberOfProcesses(array $data)
    {
        return [$data['aNproc'] => $data['lNproc']];
    }

    /**
     * Returns an array with average vmem usage as key and vmem limit as value
     *
     * @param array $data
     * @return array
     */
    private function _getVirtualMemUsage(array $data)
    {
        //in bytes
        return [$data['aVMem'] => $data['lVMem']];
    }

    /**
     * Returns an array with average pmem usage as key and pmem limit as value
     *
     * @param array $data
     * @return array
     */
    private function _getPhysicalMemUsage(array $data)
    {
        //in bytes
        return [$data['aPMem'] => $data['lPMem']];
    }

    /**
     * Returns average cpu % as key and limit cpu % as value
     *
     * @param array $data
     * @return array
     */
    private function _getCpuUsage(array $data)
    {
        //percentage
        return [$data['aCPU'] => $data['lCPU']];
    }

    /**
     * Returns average entry processes as key and limit entry processes as value
     *
     * @param array $data
     * @return array
     */
    private function _getEntryProcesses(array $data)
    {
        return [$data['aEP'] => $data['lEP']];
    }

    /**
     * Returns average io operations as key and limit io operations as value
     *
     * @param array $data
     * @return array
     */
    private function _getIoOperations(array $data)
    {
        return [$data['aIOPS'] => $data['lIOPS']];
    }

    /**
     * Returns average io usage as key and limit io usage as value
     *
     * @param array $data
     * @return array
     */
    private function _getIoUsage(array $data)
    {
        //bytes
        return [$data['aIO'] => $data['lIO']];
    }


    /**
     * Index.
     *
     * @disabledIn ~IW::WEB
     */
    public function indexAction()
    {
        $this->getView()->assign('title', 'Cloudlinux PHP Selector');
        $this->getView()->assign('template', 'default');
    }
}
